{ **************************************************************************** }
{ *                                                                          * }
{ * DONKEY KONG II (Nintendo, Game & Watch Multi Screen) Simulator           * }
{ * by Luca "MADrigal" Antignano, February 2001 (revision: March 2022)       * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:          TImage;
    btn_game_menu_background:   TImage;

    im_backdrop:                TImage;
    im_backdrop_logo:           TImage;

    // electronic game
    im_background_open:         TImage;     im_background_closed:       TImage; {1-2}
    im_background_screen_1_top: TImage;     im_background_screen_2_top: TImage; {3-4}

    btn_game_a_down:            TImage;     btn_game_a_top:             TImage; {5-6}
    btn_game_b_down:            TImage;     btn_game_b_top:             TImage; {7-8}
    btn_time_down:              TImage;     btn_time_top:               TImage; {9-10}
    btn_acl_down:               TImage;     btn_acl_top:                TImage; {11-12}
                                            btn_alarm_top:              TImage; {13}

    btn_cross_down:             TImage;     btn_cross_top:              TImage; {14-15}

    btn_cross_down_up:          TImage;                                         {144}
    btn_cross_down_down:        TImage;                                         {145}
    btn_cross_down_left:        TImage;                                         {146}
    btn_cross_down_right:       TImage;                                         {147}

    btn_jump_down:              TImage;     btn_jump_top:               TImage; {16-17}

    btn_ctrlr_overlay_up:       TImage;                                         {18}
    btn_ctrlr_overlay_left:     TImage;                                         {19}
    btn_ctrlr_overlay_right:    TImage;                                         {20}
    btn_ctrlr_overlay_down:     TImage;                                         {21}

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage; im_panel_alarm_4: TImage; {22-25}

    im_panel_game_a: TImage; im_panel_game_b: TImage;                           {26-27}

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage; {28-30}

    im_panel_life_1: TImage; im_panel_life_2: TImage;                           {31-32}

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; {33-36}

    // numbers
    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage; {37-46}
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_kong_1: TImage; im_kong_2: TImage; im_kong_3: TImage;                    {47-49}

    {                      }  {                      }  im_junior_02_01: TImage;  im_junior_03_01: TImage;  im_junior_04_01: TImage;  im_junior_05_01: TImage;  {                      }  {                      } {50-53}
    {                      }  {                      }  im_junior_02_02: TImage;  im_junior_03_02: TImage;  im_junior_04_02: TImage;  im_junior_05_02: TImage;  {                      }  {                      } {54-57}
    {                      }  im_junior_01_03: TImage;  im_junior_02_03: TImage;  im_junior_03_03: TImage;  im_junior_04_03: TImage;  im_junior_05_03: TImage;  {                      }  {                      } {58-62}
    {                      }  im_junior_01_04: TImage;  im_junior_02_04: TImage;  im_junior_03_04: TImage;  im_junior_04_04: TImage;  im_junior_05_04: TImage;  {                      }  {                      } {63-67}
    {                      }  im_junior_01_05: TImage;  im_junior_02_05: TImage;  im_junior_03_05: TImage;  im_junior_04_05: TImage;  {                      }  {                      }  {                      } {68-71}
    {                      }  im_junior_01_06: TImage;  im_junior_02_06: TImage;  im_junior_03_06: TImage;  im_junior_04_06: TImage;  im_junior_05_06: TImage;  {                      }  {                      } {72-76}
    {                      }  im_junior_01_07: TImage;  im_junior_02_07: TImage;  im_junior_03_07: TImage;  im_junior_04_07: TImage;  im_junior_05_07: TImage;  {                      }  {                      } {77-81}
    {                      }  im_junior_01_08: TImage;  im_junior_02_08: TImage;  im_junior_03_08: TImage;  im_junior_04_08: TImage;  im_junior_05_08: TImage;  {                      }  {                      } {82-86}
    {                      }  im_junior_01_09: TImage;  {                      }  {                      }  {                      }  {                      }  {                      }  {                      } {87}

                              im_junior_01_03_a: TImage;                        {88}
                              im_junior_01_03_b: TImage;                        {89}

    {                      }  im_crow_01_01: TImage;    im_crow_02_01: TImage;    im_crow_03_01: TImage;    im_crow_04_01: TImage;    im_crow_05_01: TImage;    im_crow_06_01: TImage;    {                      } {90-95}
    {                      }  {                      }  im_crow_02_02: TImage;    im_crow_03_02: TImage;    im_crow_04_02: TImage;    im_crow_05_02: TImage;    im_crow_06_02: TImage;    {                      } {96-100}
    {                      }  {                      }  im_crow_02_03: TImage;    im_crow_03_03: TImage;    im_crow_04_03: TImage;    im_crow_05_03: TImage;    im_crow_06_03: TImage;    {                      } {101-105}
    {                      }  im_snapjaw_01_04: TImage; im_snapjaw_02_04: TImage; im_snapjaw_03_04: TImage; im_snapjaw_04_04: TImage; im_snapjaw_05_04: TImage; im_snapjaw_06_04: TImage; im_snapjaw_07_04: TImage; {106-112}
    {                      }  im_spark_01_05: TImage;   im_spark_02_05: TImage;   im_spark_03_05: TImage;   im_spark_04_05: TImage;   im_spark_05_05: TImage;   {                      }  {                      } {113-117}
    im_spark_00_06: TImage;   im_spark_01_06: TImage;   im_spark_02_06: TImage;   im_spark_03_06: TImage;   im_spark_04_06: TImage;   im_spark_05_06: TImage;   im_spark_06_06: TImage;   {                      } {118-124}
    {                      }  im_snapjaw_01_08: TImage; im_snapjaw_02_08: TImage; im_snapjaw_03_08: TImage; im_snapjaw_04_08: TImage; im_snapjaw_05_08: TImage; im_snapjaw_06_08: TImage; im_snapjaw_07_08: TImage; {125-131}

    im_key_00_a: TImage; im_key_00_b: TImage; im_key_00_d: TImage; im_key_00_c: TImage; {132-135}

    im_key_02: TImage;   im_key_03: TImage;   im_key_04: TImage;   im_key_05: TImage; {136-139}

    im_chain_02: TImage; im_chain_03: TImage; im_chain_04: TImage; im_chain_05: TImage; {140-143}

{ * TIMERS                                                                   * }
    Timer_Bonus_Chance:   TTimer;
    Timer_Bonus_Complete: TTimer;
    Timer_Bonus_Lock:     TTimer;
    Timer_Clock:          TTimer;
    Timer_Game:           TTimer;
    Timer_Game_Over:      TTimer;
    Timer_Game_Start:     TTimer;
    Timer_Junior_Jump:    TTimer;
    Timer_Junior_Move:    TTimer;
    Timer_Key_Blink:      TTimer;
    Timer_Key_Move:       TTimer;
    Timer_Miss:           TTimer;
    Timer_Score_Blink:    TTimer;
    Timer_Spark_Blink:    TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Chains           (b: boolean);
    procedure Vid_Show_All_Enemies          (b: boolean);
    procedure Vid_Show_All_Junior           (b: boolean);
    procedure Vid_Show_All_Key_Zero         (b: boolean);
    procedure Vid_Show_All_Keys             (b: boolean);
    procedure Vid_Show_All_Kong             (b: boolean);
    procedure Vid_Show_All_Panel_Alarm      (b: boolean);

    procedure Vid_Show_Panel_Lives          (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

    procedure Vid_Show_Junior               (var p, q: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_background_closedClick     (Sender: TObject);

    procedure im_background_openMouseDown   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_background_openMouseMove   (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_jump_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_jump_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_upMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_upMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseDown(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseDown(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_right_down;               procedure Inp_right_up;
    procedure Inp_down_down;                procedure Inp_down_up;
    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_up_down;                  procedure Inp_up_up;

    procedure Inp_jump_down;                procedure Inp_jump_up;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Junior_Jumps_In;
    procedure Gam_Set_Mode;
    procedure Gam_Set_Timer_Game_Interval;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_Bonus_ChanceTimer       (Sender: TObject);
    procedure Timer_Bonus_CompleteTimer     (Sender: TObject);
    procedure Timer_Bonus_LockTimer         (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_Junior_JumpTimer        (Sender: TObject);
    procedure Timer_Junior_MoveTimer        (Sender: TObject);
    procedure Timer_Key_BlinkTimer          (Sender: TObject);
    procedure Timer_Key_MoveTimer           (Sender: TObject);
    procedure Timer_MissTimer               (Sender: TObject);
    procedure Timer_Score_BlinkTimer        (Sender: TObject);
    procedure Timer_Spark_BlinkTimer        (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Donkey Kong II (Nintendo, Game & Watch Multi Screen)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.01';
  HH2_CONTENT_DATE    = '2022/03/14';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Graphics improvements *';// current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 14th March, 2022';   // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     =  'Software\MADrigal\DKong250';                                  // registry key

  HINT_CLICK  = 'click or press';
  HINT_OPEN   = 'to open the game';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';
  HINT_CROSS  = 'activate the controller overlays or use';

  TIMER_JUNIOR_DELAY_SHORT = 90;
  TIMER_JUNIOR_DELAY_LONG = 330;

{}MENU_ITEMS = 14;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_chain,
  pfs_chance,
  pfs_enemy_1,
  pfs_enemy_2,
  pfs_gameover,
  pfs_jump,
  pfs_key,
  pfs_miss,
  pfs_safe,
  pfs_step:                PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time
  aIm_panel_alarm:         array[1..4]       of TImage;                         // alarm animation
  aIm_panel_lives:         array[1..2]       of TImage;                         // lives (life 0 is Junior(1,9))

  aIm_junior:              array[1..5, 1..9] of TImage;                         // Junior (i = row; j = column)
  aIm_enemies:             array[0..7, 1..8] of TImage;                         // enemies (i = row; j = column)
  aIm_kong:                array[1..3]       of TImage;                         // Donkey Kong (in chains, jumping, landed)
  aIm_chains,                                                                   // chains and locks
  aIm_keys:                array[2..5]       of TImage;                         // keys (under locks)
  aIm_key_zero:            array[1..4]       of TImage;                         // key (zero position)

{ * arrays of mixed types *}
  aEnemies:                array[0..7, 1..8] of integer;                        // enemies (i = row; j = column)

  aKeys:                   array[1..9]       of record                          // keys management (values)
                                                  int:  integer;
                                                  word: word;
                                                end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=closed game, 1=ACL-1, 2=ACL-2, 3=time, etc.
  iScore,                                                                       // 0-999
  iScoreMultiplier,                                                             // 1 (normal) or 2 (chance-time)
  iScoreChanceNext,                                                             // 300, 1300, 2300, etc.
  iHiScoreA,                                                                    // 0-999
  iHiScoreB,                                                                    // 0-999
  iLives,                                                                       // 0-2
  iPosX, iPosY,                                                                 // X = 1-5; Y = 1-9
  iKeyPosition                                                                  // 0 (low), 1 (high), 2-5 (locks), 6 (back to low)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bController,                                                                  // true while player can actually operate the main character
  bArrowKeyPressed,                                                             // true while player keeps any arrow key pressed
  bJumpKeyPressed                                                               // true while player keeps jump key pressed
                         : boolean;

  // Game variables: loops
  iLoopBonusChance,
  iLoopBonusChanceReprise,                                                      // which timer was stopped at 300 points score? 1=Timer_Bonus_Complete, 2=Timer_Bonus_Lock, 3=Timer_Key_Move, 4=Timer_Game
  iLoopBonusComplete,
  iLoopBonusLock,
  iLoopBonusLockCounter,
  iLoopBonusKeySix,
  iLoopGame,
  iLoopGameCounter,
  iLoopGameOver,
  iLoopGameStart,
  iLoopJuniorMove,                                                              // 1-4, used in Timer_Junior_Move, Junior goes 1=left, 2=up, 3=right, 4=down
  iLoopKeyMove,
  iLoopMiss,
  iLoopScoreBlink,                                                              // 1-2, used in Timer_Score_Blink
  iLoopSparkBlink                                                               // 1-2, used in Timer_Spark_Blink
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3; aIm_panel_alarm[4] := im_panel_alarm_4;
  aIm_panel_lives[1] := im_panel_life_1; aIm_panel_lives[2] := im_panel_life_2;
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  {                                   }  {                                 }    aIm_junior[2, 1] := im_junior_02_01;   aIm_junior[3, 1] := im_junior_03_01;   aIm_junior[4, 1] := im_junior_04_01;   aIm_junior[5, 1] := im_junior_05_01;   {                                   }  {                                   }
  {                                   }  {                                 }    aIm_junior[2, 2] := im_junior_02_02;   aIm_junior[3, 2] := im_junior_03_02;   aIm_junior[4, 2] := im_junior_04_02;   aIm_junior[5, 2] := im_junior_05_02;   {                                   }  {                                   }
  {                                   }  aIm_junior[1, 3] := im_junior_01_03;   aIm_junior[2, 3] := im_junior_02_03;   aIm_junior[3, 3] := im_junior_03_03;   aIm_junior[4, 3] := im_junior_04_03;   aIm_junior[5, 3] := im_junior_05_03;   {                                   }  {                                   }
  {                                   }  aIm_junior[1, 4] := im_junior_01_04;   aIm_junior[2, 4] := im_junior_02_04;   aIm_junior[3, 4] := im_junior_03_04;   aIm_junior[4, 4] := im_junior_04_04;   aIm_junior[5, 4] := im_junior_05_04;   {                                   }  {                                   }
  {                                   }  aIm_junior[1, 5] := im_junior_01_05;   aIm_junior[2, 5] := im_junior_02_05;   aIm_junior[3, 5] := im_junior_03_05;   aIm_junior[4, 5] := im_junior_04_05;   {                                   }  {                                   }  {                                   }
  {                                   }  aIm_junior[1, 6] := im_junior_01_06;   aIm_junior[2, 6] := im_junior_02_06;   aIm_junior[3, 6] := im_junior_03_06;   aIm_junior[4, 6] := im_junior_04_06;   aIm_junior[5, 6] := im_junior_05_06;   {                                   }  {                                   }
  {                                   }  aIm_junior[1, 7] := im_junior_01_07;   aIm_junior[2, 7] := im_junior_02_07;   aIm_junior[3, 7] := im_junior_03_07;   aIm_junior[4, 7] := im_junior_04_07;   aIm_junior[5, 7] := im_junior_05_07;   {                                   }  {                                   }
  {                                   }  aIm_junior[1, 8] := im_junior_01_08;   aIm_junior[2, 8] := im_junior_02_08;   aIm_junior[3, 8] := im_junior_03_08;   aIm_junior[4, 8] := im_junior_04_08;   aIm_junior[5, 8] := im_junior_05_08;   {                                   }  {                                   }
  {                                   }  aIm_junior[1, 9] := im_junior_01_09;   {                                   }  {                                   }  {                                   }  {                                   }  {                                   }  {                                   }

  {                                   }  aIm_enemies[1, 1] := im_crow_01_01;    aIm_enemies[2, 1] := im_crow_02_01;    aIm_enemies[3, 1] := im_crow_03_01;    aIm_enemies[4, 1] := im_crow_04_01;    aIm_enemies[5, 1] := im_crow_05_01;    aIm_enemies[6, 1] := im_crow_06_01;    {                                   }
  {                                   }  {                                   }  aIm_enemies[2, 2] := im_crow_02_02;    aIm_enemies[3, 2] := im_crow_03_02;    aIm_enemies[4, 2] := im_crow_04_02;    aIm_enemies[5, 2] := im_crow_05_02;    aIm_enemies[6, 2] := im_crow_06_02;    {                                   }
  {                                   }  {                                   }  aIm_enemies[2, 3] := im_crow_02_03;    aIm_enemies[3, 3] := im_crow_03_03;    aIm_enemies[4, 3] := im_crow_04_03;    aIm_enemies[5, 3] := im_crow_05_03;    aIm_enemies[6, 3] := im_crow_06_03;    {                                   }
  {                                   }  aIm_enemies[1, 4] := im_snapjaw_01_04; aIm_enemies[2, 4] := im_snapjaw_02_04; aIm_enemies[3, 4] := im_snapjaw_03_04; aIm_enemies[4, 4] := im_snapjaw_04_04; aIm_enemies[5, 4] := im_snapjaw_05_04; aIm_enemies[6, 4] := im_snapjaw_06_04; aIm_enemies[7, 4] := im_snapjaw_07_04;
  {                                   }  aIm_enemies[1, 5] := im_spark_01_05;   aIm_enemies[2, 5] := im_spark_02_05;   aIm_enemies[3, 5] := im_spark_03_05;   aIm_enemies[4, 5] := im_spark_04_05;   aIm_enemies[5, 5] := im_spark_05_05;   {                                   }  {                                   }
  aIm_enemies[0, 6] := im_spark_00_06;   aIm_enemies[1, 6] := im_spark_01_06;   aIm_enemies[2, 6] := im_spark_02_06;   aIm_enemies[3, 6] := im_spark_03_06;   aIm_enemies[4, 6] := im_spark_04_06;   aIm_enemies[5, 6] := im_spark_05_06;   aIm_enemies[6, 6] := im_spark_06_06;   {                                   }
  {                                   }  aIm_enemies[1, 8] := im_snapjaw_01_08; aIm_enemies[2, 8] := im_snapjaw_02_08; aIm_enemies[3, 8] := im_snapjaw_03_08; aIm_enemies[4, 8] := im_snapjaw_04_08; aIm_enemies[5, 8] := im_snapjaw_05_08; aIm_enemies[6, 8] := im_snapjaw_06_08; aIm_enemies[7, 8] := im_snapjaw_07_08;

  aIm_kong[1] := im_kong_1; aIm_kong[2] := im_kong_2; aIm_kong[3] := im_kong_3;

  {                                   }  {                                   }  aIm_chains[2] := im_chain_02;          aIm_chains[3] := im_chain_03;          aIm_chains[4] := im_chain_04;          aIm_chains[5] := im_chain_05;          {                                   }  {                                   }
  {                                   }  {                                   }  aIm_keys[2] := im_key_02;              aIm_keys[3] := im_key_03;              aIm_keys[4] := im_key_04;              aIm_keys[5] := im_key_05;              {                                   }  {                                   }

  aIm_key_zero[1] := im_key_00_a; aIm_key_zero[2] := im_key_00_b; aIm_key_zero[3] := im_key_00_c; aIm_key_zero[4] := im_key_00_d;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 37);                             // "Up"     (default: "UP")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 39);                             // "Left"   (default: "LEFT")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 40);                             // "Right"  (default: "RIGHT")
  aKeys[8].int := iReg.ReadInteger('', 'Key8', 38);                             // "Down"   (default: "DOWN")
  aKeys[9].int := iReg.ReadInteger('', 'Key9', 42);                             // "Jump"   (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 9 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  im_background_closed.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_OPEN;   // "Open"

  btn_game_a_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint             := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint           := HINT_ALARM;                                                     // "Alarm"

  btn_cross_top.Hint           := HINT_CROSS + ' ' + aKeyHints[aKeys[5].int] + ', '               // "Up"
                                                   + aKeyHints[aKeys[6].int] + ', '               // "Left"
                                                   + aKeyHints[aKeys[7].int] + ', '               // "Right"
                                                   + aKeyHints[aKeys[8].int];                     // "Down"

  btn_jump_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[9].int];                     // "Jump"

  btn_ctrlr_overlay_up.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Up"
  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];                     // "Right"
  btn_ctrlr_overlay_down.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[8].int];                     // "Down"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}aMenuItem[12] := pm_12_controller;
{}aMenuItem[13] := pm_13_n3;
{}aMenuItem[14] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // inits variables
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bController := false;                                                         // prevents hero movement
  bArrowKeyPressed := false;                                                    // no arrow key is currently being kept pressed
  bJumpKeyPressed  := false;                                                    // jump key is not currently being kept pressed

  Randomize;                                                                    // initializes the random numbers generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_chain);    FSOUND_Sample_Free(pfs_chance);
  FSOUND_Sample_Free(pfs_enemy_1);  FSOUND_Sample_Free(pfs_enemy_2);
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_jump);
  FSOUND_Sample_Free(pfs_key);      FSOUND_Sample_Free(pfs_miss);
  FSOUND_Sample_Free(pfs_safe);     FSOUND_Sample_Free(pfs_step);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_up.Visible    := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_down.Visible  := bDisplayCtrlOverlays and im_background_open.Visible;

  btn_cross_top.Visible := (not bDisplayCtrlOverlays) and im_background_open.Visible;
end;





procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[1].Picture := nil;

  im_panel_game_a.Visible     := b;
  im_panel_game_b.Visible     := b;
  im_panel_time_am.Visible    := b;
  im_panel_time_pm.Visible    := false;
  im_panel_time_colon.Visible := b;

  i := Ord(b) * 2; Vid_Show_Panel_Lives(i);

  Vid_Show_All_Panel_Alarm(b);
  Vid_Show_All_Kong(b);
  Vid_Show_All_Junior(b);
  Vid_Show_All_Enemies(b);
  Vid_Show_All_Key_Zero(b);
  Vid_Show_All_Keys(b);
  Vid_Show_All_Chains(b);
end;





procedure TForm1.Vid_Show_All_Chains(b: boolean);
{ displays/hides all chains and locks                                          }
  var
    i: integer;
begin
  for i := 2 to 5 do
    aIm_chains[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Enemies(b: boolean);
{ displays/hides all enemies sprites (snapjaws, sparks, crows)                 }
  var
    i, j: integer;
begin
  for j := 1 to 3 do
    for i := 2 to 6 do
      aIm_enemies[i, j].Visible := b;

  for i := 1 to 7 do
    aIm_enemies[i, 4].Visible := b;

  for j := 5 to 6 do
    for i := 1 to 5 do
      aIm_enemies[i, j].Visible := b;

  for i := 1 to 7 do
    aIm_enemies[i, 8].Visible := b;

  aIm_enemies[1, 1].Visible := b;
  aIm_enemies[0, 6].Visible := b;
  aIm_enemies[6, 6].Visible := b;
end;





procedure TForm1.Vid_Show_All_Junior(b: boolean);
{ displays/hides all Junior sprites                                            }
  var
    i, j: integer;
begin
  for j := 1 to 2 do
    for i := 2 to 5 do
      aIm_junior[i, j].Visible := b;

  for j := 3 to 4 do
    for i := 1 to 5 do
      aIm_junior[i, j].Visible := b;

  for i := 1 to 4 do
    aIm_junior[i, 5].Visible := b;

  for j := 6 to 8 do
    for i := 1 to 5 do
      aIm_junior[i, j].Visible := b;

  aIm_junior[1, 9].Visible := b;

  im_junior_01_03_a.Visible := b;
  im_junior_01_03_b.Visible := b;
end;





procedure TForm1.Vid_Show_All_Key_Zero(b: boolean);
{ displays/hides all keys in "zero position"                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_key_zero[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Keys(b: boolean);
{ displays/hides all keys (under locks)                                        }
  var
    i: integer;
begin
  for i := 2 to 5 do
    aIm_keys[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Kong(b: boolean);
{ displays/hides all Donkey Kong sprites                                       }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_kong[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Panel_Alarm(b: boolean);
{ displays/hides all alarm sprites                                             }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_alarm[i].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Lives(var p: integer);
{ displays/hides lives made (0 to 2)                                           }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_panel_lives[i].Visible := p >= i;
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;                                        // thousands (always hidden)

  if b then                                                                     // displays or hides score/high-score?
    begin                                                                         // displays
      if t > 99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div 100 mod 10].picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >  9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div  10 mod 10].picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                     aIm_panel_digit_pos[4].Picture := aIm_panel_number[t         mod 10].picture;                                            // units
    end
  else                                                                            // hides
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].picture := nil;
end;





procedure TForm1.Vid_Show_Junior(var p, q: integer; b: boolean);
{ displays/hides Junior in 'p, q' position                                     }
begin
  aIm_junior[p, q].Visible := b;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_background_closedClick(Sender: TObject);
{ opening the handheld game                                                    }
begin
  // background
  im_background_closed.Visible := false;
  im_background_open.Visible   := true;

  im_background_screen_1_top.Visible := true;
  im_background_screen_2_top.Visible := true;

  // buttons
  btn_game_a_top.Visible := true;
  btn_game_b_top.Visible := true;
  btn_time_top.Visible   := true;
  btn_acl_top.Visible    := true;
  btn_alarm_top.Visible  := true;

  btn_cross_top.Visible  := true;
  btn_jump_top.Visible   := true;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // sets mode to "ACL/2"
  iMode := 2; Gam_Set_Mode;
end;





procedure TForm1.im_background_openMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_background_openMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_jump_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ JUMP button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_jump_down;
end;





procedure TForm1.btn_jump_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ JUMP button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_jump_up;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_up_up;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;

end;





procedure TForm1.btn_ctrlr_overlay_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_down_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_acl_down    else                              // ACL
  if Key = aKeys[5].word then Inp_up_down     else                              // Up
  if Key = aKeys[6].word then Inp_left_down   else                              // Left
  if Key = aKeys[7].word then Inp_right_down  else                              // Right
  if Key = aKeys[8].word then Inp_down_down   else                              // Down
  if Key = aKeys[9].word then Inp_jump_down;                                    // Jump
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL
  if Key = aKeys[5].word then Inp_up_up     else                                // Up
  if Key = aKeys[6].word then Inp_left_up   else                                // Left
  if Key = aKeys[7].word then Inp_right_up  else                                // Right
  if Key = aKeys[8].word then Inp_down_up   else                                // Down
  if Key = aKeys[9].word then Inp_jump_up;                                      // Jump
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ Game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> HiScore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 5 then                                                             // HiScore A ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> Game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> HiScore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 7 then                                                             // HiScore B ->
    begin
      iMode := 8; Gam_Set_Mode;                                                   //         -> Game B
    end;
end;





procedure TForm1.Inp_time_down;
{ time button: PRESS                                                           }
begin
  btn_time_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //         -> Alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 3 then                                                             // Alarm ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //     -> Time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true and im_background_open.Visible;

  if iMode = 0 then                                                             // closed handheld game ->
    im_background_closedClick(Self)                                             //                      -> open handheld game
  else                                                                          // open handheld game ->
    begin
      iMode := 1; Gam_Set_Mode;                                                   //                  -> ACL/1
    end;
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  if iMode = 1 then                                                             // ACL/1 ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //     -> ACL/2
    end;
end;





procedure TForm1.Inp_up_down;
{ cross button (up): PRESS                                                     }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_up.Picture;
    end;

  if not bArrowKeyPressed then                                                  // if no arrow key is currently being pressed
    begin
      bArrowKeyPressed := true;                                                   // an arrow key has now been pressed

      if bController then                                                         // if ingame
        begin
          iLoopJuniorMove := 4;                                                     // Junior's next step is to the right
          Timer_Junior_Move.Interval := TIMER_JUNIOR_DELAY_SHORT;                   // Junior moves in 90 msec.
          Timer_Junior_Move.Enabled := true;                                        // executes move
        end;
    end;
end;





procedure TForm1.Inp_up_up;
{ cross button (up): RELEASE                                                   }
begin
  btn_cross_down.Visible := false;                                              // button animation

  bArrowKeyPressed := false;                                                    // an arrow key has been released
  iLoopJuniorMove := 0;                                                         // Junior stops moving
end;





procedure TForm1.Inp_down_down;
{ cross button (down): PRESS                                                   }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_down.Picture;
    end;

  if not bArrowKeyPressed then                                                  // if no arrow key is currently being pressed
    begin
      bArrowKeyPressed := true;                                                   // an arrow key has now been pressed

      if bController then                                                         // if ingame
        begin
          iLoopJuniorMove := 2;                                                     // Junior's next step is to the right
          Timer_Junior_Move.Interval := TIMER_JUNIOR_DELAY_SHORT;                   // Junior moves in 90 msec.
          Timer_Junior_Move.Enabled := true;                                        // executes move
        end;
    end;
end;





procedure TForm1.Inp_down_up;
{ cross button (down): RELEASE                                                 }
begin
  btn_cross_down.Visible := false;

  bArrowKeyPressed := false;                                                    // an arrow key has been released
  iLoopJuniorMove := 0;                                                         // Junior stops moving
end;





procedure TForm1.Inp_left_down;
{ cross button (left): PRESS                                                   }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_left.Picture;
    end;

  if not bArrowKeyPressed then                                                  // if no arrow key is currently being pressed
    begin
      bArrowKeyPressed := true;                                                   // an arrow key has now been pressed

      if bController then                                                         // if ingame
        begin
          iLoopJuniorMove := 3;                                                     // Junior's next step is to the right
          Timer_Junior_Move.Interval := TIMER_JUNIOR_DELAY_SHORT;                   // Junior moves in 90 msec.
          Timer_Junior_Move.Enabled := true;                                        // executes move
        end;
    end;
end;





procedure TForm1.Inp_left_up;
{ cross button (left): RELEASE                                                 }
begin
  btn_cross_down.Visible := false;

  bArrowKeyPressed := false;                                                    // an arrow key has been released
  iLoopJuniorMove := 0;                                                         // Junior stops moving
end;





procedure TForm1.Inp_right_down;
{ cross button (right): PRESS                                                  }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_right.Picture;
    end;

  if not bArrowKeyPressed then                                                  // if no arrow key is currently being pressed
    begin
      bArrowKeyPressed := true;                                                   // an arrow key has now been pressed

      if bController then                                                         // if ingame
        begin
          iLoopJuniorMove := 1;                                                     // Junior's next step is to the right
          Timer_Junior_Move.Interval := TIMER_JUNIOR_DELAY_SHORT;                   // Junior moves in 90 msec.
          Timer_Junior_Move.Enabled := true;                                        // executes move
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ cross button (right): RELEASE                                                }
begin
  btn_cross_down.Visible := false;

  bArrowKeyPressed := false;                                                    // an arrow key has been released
  iLoopJuniorMove := 0;                                                         // Junior stops moving
end;





procedure TForm1.Inp_jump_down;
{ jump button: PRESS                                                           }
begin
  btn_jump_down.Visible := true and im_background_open.Visible;

  if not bJumpKeyPressed then                                                   // if jump key is not being pressed
    begin
      bJumpKeyPressed := true;                                                    // jump key has now been pressed

      if bController then                                                         // if ingame
        begin
          iLoopJuniorMove := 5;                                                      // Junior jumps immediately
          Timer_Junior_Move.Interval := 1;
          Timer_Junior_Move.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_jump_up;
{ jump button: RELEASE                                                         }
begin
  btn_jump_down.Visible := false;

  bJumpKeyPressed := false;                                                     // jump key has been released
  iLoopJuniorMove := 0;                                                         // Junior stops moving
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Bonus_Chance.Enabled   := false;
  Timer_Bonus_Complete.Enabled := false;
  Timer_Bonus_Lock.Enabled     := false;
  Timer_Clock.Enabled          := false;
  Timer_Game.Enabled           := false;
  Timer_Game_Over.Enabled      := false;
  Timer_Game_Start.Enabled     := false;
  Timer_Junior_Jump.Enabled    := false;
  Timer_Junior_Move.Enabled    := false;
  Timer_Key_Blink.Enabled      := false;
  Timer_Key_Move.Enabled       := false;
  Timer_Miss.Enabled           := false;
  Timer_Score_Blink.Enabled    := false;
  Timer_Spark_Blink.Enabled    := false;
end;





procedure TForm1.Gam_Junior_Jumps_In;
{ Junior jumps out from the "pit" and goes into (1,8); lives are decreased by  }
{ one                                                                          }
begin
  Vid_Show_Junior(iPosX, iPosY, false);
  iPosX := 1; iPosY := 8;
  Vid_Show_Junior(iPosX, iPosY, true);

  Snd_Play(pfs_step);
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
begin
  case iMode of
    0: begin                                                                    // handheld is closed
       end;
    1: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bController := false;
       end;
    2: Vid_Show_All(true);                                                      // ACL/2 (ACL button release)
    3: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;
         Vid_Show_All(false);

         im_panel_time_colon.Visible := true;
         im_panel_time_pm.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
       end;
    4: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;
         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;
       end;
    5, 7: begin                                                                 // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         case iMode of
           5: begin
                im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true);
              end;
           7: begin
                im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true);
              end;
         end; // case..of

         iLives := 2; Vid_Show_Panel_Lives(iLives);

         iPosX := 1; iPosY := 9; Vid_Show_Junior(iPosX, iPosY, true);
       end;
    6, 8: begin                                                                 // GAME A/B (Game A/B button release)
         iScore := 0; Vid_Show_Panel_Score(iScore, true);

         iLoopGameStart := 1;
         Timer_Game_Start.Interval := 1;
         Timer_Game_Start.Enabled := true;
       end;
    9: begin                                                                    // GAME OVER
         // do nothing
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Timer_Game_Interval;
{ calculates Timer_Game.Interval depending on iScore and iLoopGameCounter      }
  var
    i: integer;
begin
  case iMode of
    6: i := 600 - (iLoopGameCounter)               - (iScore div 3);            // Game A
    8: i := 600 - ((iLoopGameCounter * 13) div 10) - (iScore div 3);            // Game B
  end; // case..of

  if i < 120 then
    i := 120;

  Timer_Game.Interval := i;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Bonus_ChanceTimer(Sender: TObject);
{ Junior has opened a lock                                                     }
  var
    i: integer;
begin
  case iLoopBonusChance of
    1: begin
         if iLives < 2 then
           Inc(iLives)
         else
           iScoreMultiplier := 2;

         Timer_Bonus_Chance.Interval := 270;
       end;
    2: begin Snd_Play(pfs_chance); i := iLives - 1; Vid_Show_Panel_Lives(i); end;
    3:                                              Vid_Show_Panel_Lives(iLives);
    4: begin                       i := iLives - 1; Vid_Show_Panel_Lives(i); end;
    5:                                              Vid_Show_Panel_Lives(iLives);
    6: begin                       i := iLives - 1; Vid_Show_Panel_Lives(i); end;
    7: begin
         Vid_Show_Panel_Lives(iLives);

         if iScoreMultiplier = 2 then
           begin
             iLoopScoreBlink := 1;
             Timer_Score_Blink.Interval := 500;
             Timer_Score_Blink.Enabled := true;
           end;

         Inc(iScoreChanceNext, 1000);                                           // sets next bonus score (+1000 points)

         // which timer reprises?
         case iLoopBonusChanceReprise of
           1: Timer_Bonus_Complete.Enabled := true;
           2: Timer_Bonus_Lock.Enabled := true;
           3: Timer_Key_Move.Enabled := true;
           4: begin
                Timer_Game.Enabled := true;
                bController := true;

                Timer_Key_Blink.Interval := 500;
                Timer_Key_Blink.Enabled := true;

                iLoopSparkBlink := 1;
                Timer_Spark_Blink.Interval := 50;
                Timer_Spark_Blink.Enabled := true;
              end;
         end; // case..of

         Timer_Bonus_Chance.Enabled:=false;
       end;
  end; // case..of

  Inc(iLoopBonusChance);
end;





procedure TForm1.Timer_Bonus_CompleteTimer(Sender: TObject);
{ Donkey Kong has been rescued by Junior                                       }
begin
  case iLoopBonusComplete of
    1, 3, 5, 7, 9, 11, 13: begin
         Vid_Show_Junior(iPosX, iPosY, false);
         if (iPosY < 4) then
           Inc(iPosY)
         else
           if (iPosX < 5) then
             Inc(iPosX)
           else
             iLoopBonusComplete := 19;                                          // becomes 20 immediately

         Vid_Show_Junior(iPosX, iPosY, true);

         aIm_kong[1].Visible := false;
         Snd_Play(pfs_step);

         Inc(iLoopBonusComplete);

         Timer_Bonus_Complete.Interval := 200;
       end;
    2, 4, 6, 8, 10, 12: begin
         aIm_kong[1].Visible := true;

         Inc(iLoopBonusComplete);

         Timer_Bonus_Complete.Interval := 340;
       end;
   20: begin
         aIm_kong[1].Visible := false;
         aIm_kong[2].Visible := true;
         Snd_Play(pfs_safe);

         Inc(iLoopBonusComplete);

         Timer_Bonus_Complete.Interval := 790;
       end;
   21: begin
         Vid_Show_Junior(iPosX, iPosY, false);
         aIm_kong[2].Visible := false;
         aIm_kong[3].Visible := true;

         Inc(iLoopBonusComplete);

         Timer_Bonus_Complete.Interval := 1100;
       end;
   22..41: begin
         Inc(iScore, iScoreMultiplier);
         Vid_Show_Panel_Score(iScore, true);

         Snd_Play(pfs_key);

         // checks for "bonus score"
         if iScore >= iScoreChanceNext then
           begin
             Gam_Stop_All_Timers;
             Vid_Show_Panel_Score(iScore, true);                                // forces score display in case it was blinking
             Snd_Silence;
             bController := false;

             iLoopBonusChanceReprise := 1;                                      // needed in Timer_Bonus_Chance

             iLoopBonusChance := 1;
             Timer_Bonus_Chance.Interval := 1;
             Timer_Bonus_Chance.Enabled := true;
           end;

         // loops timer
         Inc(iLoopBonusComplete);

         Timer_Bonus_Complete.Interval := 95;
       end;
   42: begin
         Inc(iLoopBonusComplete);

         Timer_Bonus_Complete.Interval := 720;
       end;

   43: begin
         aIm_kong[3].Visible := false;
         aIm_kong[1].Visible := true;
         iPosX := 1; iPosY := 9; Vid_Show_Junior(iPosX, iPosY, true);
         Snd_Play(pfs_key);

         Inc(iLoopBonusComplete);
       end;
   44: begin
         Snd_Play(pfs_chain);
         Vid_Show_All_Chains(true);

         iLoopGameCounter := 1;
         Gam_Set_Timer_Game_Interval;
         Timer_Game.Enabled := true;

         iKeyPosition := 0;
         Timer_Key_Blink.Interval := 500;
         Timer_Key_Blink.Enabled:=true;

         iLoopSparkBlink := 1;
         Timer_Spark_Blink.Interval := 50;
         Timer_Spark_Blink.Enabled := true;

         bController := true;
         bArrowKeyPressed := false;

         Timer_Bonus_Complete.Enabled := false;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Bonus_LockTimer(Sender: TObject);
{ a key was placed in a lock                                                   }
  var
    i, j: integer;
    bLocks: boolean;
begin
  case iLoopBonusLock of
    1: begin
         aIm_keys[iKeyPosition].Visible := true;
         Snd_Play(pfs_key);

         // sets bonus score
         iLoopBonusLockCounter := 15;

         while iLoopGameCounter > 24 do
           begin
             Dec(iLoopGameCounter, 12);
             Dec(iLoopBonusLockCounter);
           end;

         if iLoopBonusLockCounter < 5 then
           iLoopBonusLockCounter := 5;

         Timer_Bonus_Lock.Interval := 290;

         Inc(iLoopBonusLock);
       end;
    2, 3, 4: begin
         aIm_keys[iKeyPosition].Visible := Odd(iLoopBonusLock);
         aIm_chains[iKeyPosition].Visible := Odd(iLoopBonusLock);
         Snd_Play(pfs_key);

         Inc(iLoopBonusLock);
       end;
    5: begin
         Inc(iScore, iScoreMultiplier);
         Vid_Show_Panel_Score(iScore, true);

         Snd_Play(pfs_key);

         // checks for "bonus score"
         if iScore >= iScoreChanceNext then
           begin
             Gam_Stop_All_Timers;
             Vid_Show_Panel_Score(iScore, true);                                // forces score display in case it was blinking
             Snd_Silence;
             bController := false;

             iLoopBonusChanceReprise := 2;                                      // needed in Timer_Bonus_Chance

             iLoopBonusChance := 1;
             Timer_Bonus_Chance.Interval := 1;
             Timer_Bonus_Chance.Enabled := true;
           end;

         // loops timer
         Dec(iLoopBonusLockCounter);

         if iLoopBonusLockCounter > 0 then
           Timer_Bonus_Lock.Interval := 95
         else
           Inc(iLoopBonusLock);
       end;
    6: begin
         // clears upper screen in Game A
         if (iMode = 6) then
           begin
             for i := 2 to 6 do
               for j := 1 to 4 do
                 begin
                   aEnemies[i, j] := 0;
                   aIm_enemies[i, j].Visible := false;
                 end;

             aEnemies[1, 4] := 0;
             aIm_enemies[1, 4].Visible := false;
           end;

         // checks how many locks still to open
         bLocks := false;

         for i := 2 to 5 do
           if aIm_chains[i].Visible then
             bLocks := true;

         if bLocks then
           begin
             // game continues
             Timer_Game.Enabled := true;

             iKeyPosition := 6;
             Timer_Key_Blink.Interval := 500;
             Timer_Key_Blink.Enabled:=true;

             iLoopSparkBlink := 1;
             Timer_Spark_Blink.Interval := 50;
             Timer_Spark_Blink.Enabled := true;

             bController := true;
             bArrowKeyPressed := false;
           end
         else
           begin
             // Donkey Kong has been rescued!
             iLoopBonusComplete := 1;
             Timer_Bonus_Complete.Interval := 1;
             Timer_Bonus_Complete.Enabled := true;
           end;

          Timer_Bonus_Lock.Enabled := false;
       end;
  end; // case..of
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
var
  i, j: integer;
  bGenerate, bMiss: boolean;
begin
  // sets "has any miss occured?" to false now
  bMiss := false;

  // if Junior in (0,0) and 15 ticks occured, Junior jumps to (1, 8)
  if (iPosX = 1) and (iPosY = 9) and (iLoopGameCounter = 22) then
    Gam_Junior_Jumps_In;

  if Odd(iLoopGame) then                                                        // odd ticks
    begin
      // collision check (part 1)
      if ( ((iPosY = 1) or (iPosY = 3)) and (iPosX > 1) and (iPosX < 6) and (aEnemies[iPosX, iPosY] = 1) ) or
         ( (iPosY = 8) and (aEnemies[iPosX + 1, iPosY] = 1) ) then
        bMiss := true;

      if not bMiss then
        begin
          // enemies move (rows 1-3-5-8)
          for i := 6 downto 2 do aEnemies[i, 1] := aEnemies[i - 1, 1];
          for i := 6 downto 2 do aEnemies[i, 3] := aEnemies[i - 1, 3]; aEnemies[2, 3] := aEnemies[2, 2]; aEnemies[2, 2] := 0;
          for i := 1 to 4 do aEnemies[i, 5] := aEnemies[i + 1, 5];
          for i := 1 to 6 do aEnemies[i, 8] := aEnemies[i + 1, 8];

          // enemies generate (rows 1-5-8)
          bGenerate := false;
          if Random(13 - iMode) = 0 then bGenerate := true;
          if aEnemies[2, 1] = 1 then bGenerate := false;
          aEnemies[1, 1] := Ord(bGenerate);

          bGenerate := false;
          if Random(6) = 0 then bGenerate := true;
          if aEnemies[4, 5] = 1 then bGenerate := false;
          aEnemies[5, 5] := Ord(bGenerate);

          bGenerate := false;
          if Random(6) = 0 then bGenerate := true;
          if aEnemies[6, 8] = 1 then bGenerate := false;
          aEnemies[7, 8] := Ord(bGenerate);
        end;

      // collision check (part 2, after enemies movements)
      if (iPosX = 1) and (iPosY = 5) and (aEnemies[1, 5] = 1) then              // Junior and spark on the left vines in (1,5)
        bMiss := true;

      if (iPosX >= 2) and (iPosX <= 4) and (iPosY = 5)                          // Junior is jumping on the middle level
        and (aEnemies[iPosX, 5] = 1) then                                         // a spark is approaching from the right
          if aEnemies[iPosX + 1, 6] = 1 then                                        // if there's a spark under Junior
            begin                                                                     // force Junior to land
              // Junior lands immediately
              Timer_Junior_Jump.Interval := 1;
              Timer_Junior_Jump.Enabled := true;
            end
          else                                                                      // else
            if aEnemies[iPosX, 6] = 1 then                                            // if there's a spark in front ot to Junior
              begin                                                                     // move the snapjaws and then force Junior to land
                // enemies move (row 6)
                for i := 6 downto 1 do aEnemies[i, 6] := aEnemies[i - 1, 6];

                // enemies generate (row 6)
                bGenerate := false;
                if Random(6) = 0 then bGenerate := true;
                if aEnemies[1, 6] = 1 then bGenerate := false;
                aEnemies[0, 6] := Ord(bGenerate);

                // Junior lands immediately
                Timer_Junior_Jump.Interval := 1;
                Timer_Junior_Jump.Enabled := true;
              end                                                                       // else
            else                                                                          // a miss is made
              bMiss := true;

      // plays sound
      if not bMiss then
        if iPosY > 4 then
          Snd_Play(pfs_enemy_2)
        else
          Snd_Play(pfs_enemy_1);
    end
  else                                                                          // even ticks
    begin
      // collision check (part 1)
      if ( (iPosY = 2) and (iPosX > 1) and (iPosX < 6) and (aEnemies[iPosX + 1, iPosY] = 1) ) or
         ( (iPosY = 4) and (aEnemies[iPosX + 1, iPosY] = 1) ) or
         ( (iPosY = 6) and (aEnemies[iPosX, iPosY] = 1) ) then
        bMiss := true;

      if not bMiss then
        begin
          // enemies move (rows 2-4-6)
          for i := 2 to 5 do aEnemies[i, 2] := aEnemies[i + 1, 2]; aEnemies[6, 2] := aEnemies[6, 1]; aEnemies[6, 1] := 0;
          for i := 1 to 6 do aEnemies[i, 4] := aEnemies[i + 1, 4];
          for i := 6 downto 1 do aEnemies[i, 6] := aEnemies[i - 1, 6];

          // enemies generate (rows 4-6)
          bGenerate := false;
          if Random(6) = 0 then bGenerate := true;
          if aEnemies[7, 4] = 1 then bGenerate := false;
          aEnemies[7, 4] := Ord(bGenerate);

          bGenerate := false;
          if Random(6) = 0 then bGenerate := true;
          if aEnemies[1, 6] = 1 then bGenerate := false;
          aEnemies[0, 6] := Ord(bGenerate);
        end;

      // collision check (part 2, after enemies movements)
      if (iPosX = 5) and (iPosY = 7) and (aEnemies[5, 6] = 1) then              // Junior on the right vines (5,7)
        bMiss := true;

      if (iPosX >= 1) and (iPosX <= 4) and (iPosY = 7)                          // Junior is jumping on the lowest level
        and (aEnemies[iPosX, 6] = 1) then                                         // a spark is approaching from the left
          if aEnemies[iPosX, 8] = 1 then                                            // if there's a snapjaw under Junior
            begin                                                                     // force Junior to land
              // Junior lands immediately
              Timer_Junior_Jump.Interval := 1;
              Timer_Junior_Jump.Enabled := true;
            end
          else                                                                     // else
            if aEnemies[iPosX + 1, 8] = 1 then                                       // if there's a snapjaw in front of Junior
              begin                                                                    // move the snapjaws and then force Junior to land
                // enemies move (row 8)
                for i := 1 to 6 do aEnemies[i, 8] := aEnemies[i + 1, 8];

                // enemies generate (row 8)
                bGenerate := false;
                if Random(6) = 0 then bGenerate := true;
                if aEnemies[6, 8] = 1 then bGenerate := false;
                aEnemies[7, 8] := Ord(bGenerate);

                // Junior lands immediately
                Timer_Junior_Jump.Interval := 1;
                Timer_Junior_Jump.Enabled := true;
              end                                                                    // else
            else                                                                       // a miss is made
              bMiss := true;

      // plays sound
      if not bMiss then
        Snd_Play(pfs_enemy_2);
    end;

  // renders enemies
  Vid_Show_All_Enemies(false);

  for i := 0 to 7 do
    for j := 1 to 8 do
      if aEnemies[i, j] = 1 then
        aIm_Enemies[i, j].Visible := true;

  // eventually executes "miss" routine
  if bMiss then
    begin
      Gam_Stop_All_Timers;
      Vid_Show_Panel_Score(iScore, true);
      Snd_Silence;

      bController := false;

      iLoopMiss := 1;
      Timer_Miss.Interval := 1;
      Timer_Miss.Enabled := true;
    end;

  Gam_Set_Timer_Game_Interval;

  Inc(iLoopGame);

  if iLoopGame = 3 then
    iLoopGame := 1;

  Inc(iLoopGameCounter);
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ no lives left. plays "game over" tune, updates highscores and goes into      }
{ "time mode" after 5 minutes                                                  }
begin
  case iLoopGameOver of
    1: Timer_Game_Over.interval := 200;
    2: begin
         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 500;
       end;
    3: begin
         while iScore >= 1000 do
           Dec(iScore, 1000);

         if (iMode = 6) and (iScore > iHiScoreA) then
           begin
             iHiScoreA := iScore;
             iReg.WriteInteger('', 'HiScore1', iHiScoreA);
           end;

         if (iMode = 8) and (iScore > iHiScoreB) then
           begin
             iHiScoreB := iScore;
             iReg.WriteInteger('', 'HiScore2', iHiScoreB);
           end;

         iMode := 9; Gam_Set_Mode;

         Timer_Game_Over.Interval := 300000;
       end;
    4: begin                                                                    // goes automatically to time mode after 5 minutes
         Vid_Show_All(false);
         iMode := 3; Gam_Set_Mode;

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameOver);
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ inits all game variables                                                     }
  var
    i, j: integer;
begin
  case iLoopGameStart of
    1: Timer_Game_Start.Interval := 550;
    2: begin
         aIm_panel_alarm[1].Visible := true;
         aIm_kong[1].Visible := true;
         Snd_Play(pfs_key);
       end;
    3: begin
         Vid_Show_All_Chains(true);
         Snd_Play(pfs_chain);
       end;
    4: begin
         for i := 0 to 7 do
           for j := 1 to 8 do
             aEnemies[i, j] := 0;

         iScoreMultiplier := 1;
         iScoreChanceNext := 300;

         iLoopGame := 1;
         iLoopGameCounter := 1;
         Gam_Set_Timer_Game_Interval;
         Timer_Game.Enabled := true;

         iKeyPosition := 0;
         Timer_Key_Blink.Interval := 500;
         Timer_Key_Blink.Enabled := true;

         iLoopSparkBlink := 1;
         Timer_Spark_Blink.Interval := 50;
         Timer_Spark_Blink.Enabled := true;

         bController := true;
         bArrowKeyPressed := false;

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_Junior_JumpTimer(Sender: TObject);
{ Junior just jumped, he's landing now                                         }
begin
  Vid_Show_Junior(iPosX, iPosY, false);
  Inc(iPosY);
  Vid_Show_Junior(iPosX, iPosY, true);

  im_junior_01_03_a.Visible := false;

  if ( (iPosY = 4) and (aEnemies[iPosX, 4] = 1) ) or
     ( (iPosY = 6) and (aEnemies[iPosX + 1, 6] = 1) ) or
     ( (iPosY = 8) and (aEnemies[iPosX, 8] = 1) ) then
    begin
      Inc(iScore, iScoreMultiplier);
      Vid_Show_Panel_Score(iScore, true);

      // checks for "bonus score"
      if iScore >= iScoreChanceNext then
        begin
          Gam_Stop_All_Timers;
          Vid_Show_Panel_Score(iScore, true);                                   // forces score display in case it was blinking
          Snd_Silence;
          bController := false;

          iLoopBonusChanceReprise := 4;                                         // needed in Timer_Bonus_Chance

          iLoopBonusChance := 1;
          Timer_Bonus_Chance.Interval := 1;
          Timer_Bonus_Chance.Enabled := true;
        end
      else
        bController := true;
    end
  else
    begin
      bController := true;
      bArrowKeyPressed := false;
    end;

  Timer_Junior_Jump.Enabled := false;
end;





procedure TForm1.Timer_Junior_MoveTimer(Sender: TObject);
{ manages Junior movements depending on iLoopJuniorMove variable set before    }
{ call                                                                         }
  var
    bJumped,          // true = Junior just jumped
    bJumpIn,          // true = Junior jumps into action
    bKeyLock,         // true = a key was placed into a lock (bonus)
    bKeyMoveZero,     // true = key moved from 0 to 1
    bKeyMoveOne,      // true = key moved from 1 to 2-5
    bKeyMoveSix,      // true = key moved from 6 to 1 (that's 0 to 1 + bonus score)
    bLandNext,        // true = Junior just jumped and must land
    bMiss,            // true = a miss has occured
    bMoved: boolean;  // true = Junior just moved
begin
  bJumped      := false;
  bJumpIn      := false;
  bKeyLock     := false;
  bKeyMoveZero := false;
  bKeyMoveOne  := false;
  bKeyMoveSix  := false;
  bLandNext    := false;
  bMiss        := false;
  bMoved       := false;

  case iLoopJuniorMove of
    1: if (iPosY = 4) or                                                        // move right
          ((iPosY = 6) and (iPosX < 5)) or
          (iPosY = 8) then
         if aEnemies[iPosX + 1, iPosY] = 1 then
           bMiss := true
         else
           if iPosX < 5 then
             begin
               Vid_Show_Junior(iPosX, iPosY, false);
               Inc(iPosX);
               Vid_Show_Junior(iPosX, iPosY, true);
               bMoved := true;
             end;
    2: if ( (iPosX > 1) and (iPosY < 4) ) or                                    // move down
          ( (iPosX = 1) and ((iPosY = 4) or (iPosY = 5)) ) or
          ( (iPosX = 5) and ((iPosY = 6) or (iPosY = 7)) ) then
         begin
           Vid_Show_Junior(iPosX, iPosY, false);
           Inc(iPosY);
           Vid_Show_Junior(iPosX, iPosY, true);

           if ( ((iPosX > 1) and (iPosY = 2)) and (aEnemies[iPosX + 1, iPosY] = 1) ) or
              ( ((iPosX > 1) and (iPosY = 3)) and (aEnemies[iPosX, iPosY] = 1) ) or
              ( ((iPosX = 1) and (iPosY = 5)) and (aEnemies[iPosX, iPosY] = 1) ) or
              ( ((iPosX = 1) and (iPosY = 6)) and (aEnemies[iPosX, iPosY] = 1) ) or
              ( ((iPosX = 5) and (iPosY = 7)) and (aEnemies[5, 6] = 1) )         then
             bMiss := true
           else
             bMoved := true;
         end;
    3: if (iPosY = 4) or                                                        // move left
          (iPosY = 6) or
          (iPosY = 8) then
         if iPosX > 1 then
           begin
             Vid_Show_Junior(iPosX, iPosY, false);
             Dec(iPosX);
             Vid_Show_Junior(iPosX, iPosY, true);

             if aEnemies[iPosX + 1, iPosY] = 1 then
               bMiss := true
             else
               bMoved := true;
           end;
    4: if (iPosX = iKeyPosition) and (iPosY = 1) then                           // move up
         bKeyLock := true
       else
         if (iPosX = 1) and (iPosY = 9) then
           bJumpIn := true
         else
           if ( (iPosX > 1 ) and (iPosY > 1) and (iPosY < 4) ) or
              ( (iPosX = 1 ) and (iPosY = 5) ) or
              ( (iPosX = 5 ) and (iPosY = 7) ) then
             begin
               Vid_Show_Junior(iPosX, iPosY, false);
               Dec(iPosY);
               Vid_Show_Junior(iPosX, iPosY, true);

               if ( ((iPosX > 1) and (iPosY = 2)) and (aEnemies[iPosX + 1, iPosY] = 1) ) or
                  ( ((iPosX > 1) and (iPosY = 3)) and (aEnemies[iPosX, iPosY] = 1) ) then
                 bMiss := true
               else
                 bMoved := true;
             end;
    5: if (iPosY = 4) or
          ( (not (iPosX = 5)) and (iPosY = 6) ) or
          (iPosY = 8) then
         begin
           // displays Junior
           Vid_Show_Junior(iPosX, iPosY, false);
           Dec(iPosY);
           Vid_Show_Junior(iPosX, iPosY, true);

           if (iPosX = 1) and (iPosY = 3) then
             im_junior_01_03_a.Visible := true;

           // collision check
           if ( (iPosY = 3) and (iPosX > 1) and (aEnemies[iPosX, iPosY] = 1) ) or
              ( (iPosY = 5) and (aEnemies[iPosX, iPosY] = 1) ) or
              ( (iPosY = 7) and (aEnemies[iPosX, iPosY - 1] = 1) ) then
             bMiss := true
           else
             bJumped := true;

           // keys check
           bKeyMoveZero := (iPosX = 1) and (iPosY = 7) and (iKeyPosition = 0);
           bKeyMoveOne  := (iPosX = 1) and (iPosY = 3) and (iKeyPosition = 1);
           bKeyMoveSix  := (iPosX = 1) and (iPosY = 7) and (iKeyPosition = 6);

           // jump/land check
           if (not bMiss) and (not bKeyMoveZero) and (not bKeyMoveOne) and (not bKeyMoveSix) and
              ( not ((iPosX = 5) and (iPosY = 7)) ) and
              ( not ((iPosX = 1) and (iPosY = 5)) ) and
              ( not ((iPosX > 1) and (iPosY = 3)) ) then
             bLandNext := true;
         end;
  end; // case..of

  // checks
  if bArrowKeyPressed then                                                      // if any arrow key is still pressed
    if Timer_Game.Interval > TIMER_JUNIOR_DELAY_LONG then
      Timer_Junior_Move.Interval := TIMER_JUNIOR_DELAY_LONG                         // Junior moves again in 330 msec....
    else
      Timer_Junior_Move.Interval := Timer_Game.Interval                             // or less if game is too fast
  else
    Timer_Junior_Move.Enabled := false;                                           // else stops walking

  if bMiss then                                                                 // if a miss occurred
    begin                                                                         // executes TimerMiss procedure
      Gam_Stop_All_Timers;
      bController := false;

      Vid_Show_Panel_Score(iScore, true);                                           // forces score display in case it was blinking

      iLoopMiss := 1;
      Timer_Miss.Interval := 1;
      Timer_Miss.Enabled := true;
    end
  else                                                                          // else
    if bKeyLock then                                                              // a lock was open
      begin
        Gam_Stop_All_Timers;
        bController := false;

        iLoopBonusLock := 1;
        Timer_Bonus_Lock.Interval := 1;
        Timer_Bonus_Lock.Enabled := true;
      end
    else                                                                           // else
      if bKeyMoveZero then                                                           // key moves from 0 to 1
        begin
          Timer_Key_Blink.Enabled := false;

          iLoopKeyMove := 1;
          Timer_Key_Move.Interval := 1;
          Timer_Key_Move.Enabled := true;

          Timer_Junior_Jump.Interval := Timer_Game.Interval * 2;
          Timer_Junior_Jump.Enabled := true;
        end
      else                                                                           // else
        if bKeyMoveOne then                                                            // key moves from 1 to 2-5
          begin
            Timer_Key_Blink.Enabled := false;

            iLoopKeyMove := 4;
            Timer_Key_Move.Interval := 1;
            Timer_Key_Move.Enabled := true;

            Timer_Junior_Jump.Interval := Timer_Game.Interval * 2;
            Timer_Junior_Jump.Enabled := true;
          end
        else                                                                           // else
          if bKeyMoveSix then                                                            // key moves from 6 to 1 (0 to 1 + bonus)
            begin
              Gam_Stop_All_Timers;
              bController := false;

              iLoopKeyMove := 7;
              Timer_Key_Move.Interval := 1;
              Timer_Key_Move.Enabled := true;
            end
          else                                                                           // else
            if bJumpIn then                                                                // Junior jumps in
              Gam_Junior_Jumps_In
            else                                                                           // else
              if bLandNext then                                                              // Junior lands on next tick
                begin
                  Timer_Junior_Jump.Interval := Timer_Game.Interval * 2;
                  Timer_Junior_Jump.Enabled := true;
                end;

  // plays sound
  if not bMiss then
    if bMoved then                                                              
      Snd_Play(pfs_step)
    else
      if bJumped then
        Snd_Play(pfs_jump);
end;





procedure TForm1.Timer_Key_BlinkTimer(Sender: TObject);
{ key blinks                                                                   }
begin
  case iKeyPosition of
    0, 6: aIm_key_zero[1].Visible := not aIm_key_zero[1].Visible;               // 6 = Junior returns home after opening a lock
    1: aIm_key_zero[3].Visible := not aIm_key_zero[3].Visible;
    2: aIm_keys[2].Visible := not aIm_keys[2].Visible;
    3: aIm_keys[3].Visible := not aIm_keys[3].Visible;
    4: aIm_keys[4].Visible := not aIm_keys[4].Visible;
    5: aIm_keys[5].Visible := not aIm_keys[5].Visible;
  end; // case..of
end;





procedure TForm1.Timer_Key_MoveTimer(Sender: TObject);
{ Junior takes or uses the key. Three cases may occur: the key is moved from   }
{ the lower floor to the upper one; the key is moved fom the upper floor to    }
{ the lock; the key is returned to the lower floor                             }
begin
  case iLoopKeyMove of
    // 0 to 1
    1: Timer_Key_Move.Interval := 400;
    2: begin
         aIm_key_zero[1].Visible := false;
         aIm_key_zero[2].Visible := true;
         Snd_Play(pfs_key);
       end;
    3: begin
         aIm_key_zero[2].Visible := false;
         aIm_key_zero[3].Visible := true;
         iKeyPosition := 1;

         Timer_Key_Move.Enabled := false;

         Timer_Key_Blink.Interval := 500;
         Timer_Key_Blink.Enabled := true;
       end;
    // 1 to 2
    4: Timer_Key_Move.Interval := 400;
    5: begin
         aIm_key_zero[3].Visible := false;
         aIm_key_zero[4].Visible := true;
         Snd_Play(pfs_key);
       end;
    6: begin
         aIm_key_zero[4].Visible := false;

         repeat
           iKeyPosition := Random(4) + 2;
         until
           aIm_chains[iKeyPosition].Visible;

         Timer_Key_Blink.Interval := 500;
         Timer_Key_Blink.Enabled := true;

         Timer_Key_Move.Enabled := false;
       end;
    // 6 to 1 (0 to 1 + bonus)
    7: Timer_Key_Move.Interval := 400;
    8: begin
         aIm_key_zero[1].Visible := false;
         aIm_key_zero[2].Visible := true;
         Snd_Play(pfs_key);
       end;
    9: begin
         aIm_key_zero[2].Visible := false;
         aIm_key_zero[3].Visible := true;
         iKeyPosition := 1;

         Timer_Key_Move.Interval := 95;
       end;
   10..14: begin
         Inc(iScore, iScoreMultiplier);
         Vid_Show_Panel_Score(iScore, true);

         Snd_Play(pfs_key);

         // checks for "bonus score"
         if iScore >= iScoreChanceNext then
           begin
             Gam_Stop_All_Timers;
             Vid_Show_Panel_Score(iScore, true);                                // forces score display in case it was blinking
             Snd_Silence;
             bController := false;

             iLoopBonusChanceReprise := 3;                                      // needed in Timer_Bonus_Chance

             iLoopBonusChance := 1;
             Timer_Bonus_Chance.Interval := 1;
             Timer_Bonus_Chance.Enabled := true;
           end;
       end;
   15: begin
         Gam_Set_Timer_Game_Interval;
         Timer_Game.Enabled := true;

         iLoopSparkBlink := 1;
         Timer_Spark_Blink.Interval := 50;
         Timer_Spark_Blink.Enabled := true;

         Timer_Key_Blink.Interval := 500;
         Timer_Key_Blink.Enabled := true;

         Timer_Junior_Jump.Interval := Timer_Game.Interval * 2;
         Timer_Junior_Jump.Enabled := true;

         bController := true;
         bArrowKeyPressed := false;

         Timer_Key_Move.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopKeyMove);
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occured: Junior was hit by an enemy                               }
begin
  case iLoopMiss of
    1: begin
         Snd_Play(pfs_miss);
         Timer_Miss.Interval := 160;
       end;
    2: begin
         Vid_Show_Junior(iPosX, iPosY, false);
         Timer_Miss.Interval := 280;
       end;
    3, 5: Vid_Show_Junior(iPosX, iPosY, true);
    4, 6: Vid_Show_Junior(iPosX, iPosY, false);
    7: begin
         Dec(iLives); Vid_Show_Panel_Lives(iLives);

         if iLives >= 0 then
           begin
             iPosX := 1; iPosY := 9; Vid_Show_Junior(iPosX, iPosY, true);
             Vid_Show_All_Keys(false);
             Vid_Show_All_Key_Zero(false);

             iScoreMultiplier := 1;

             iLoopGameCounter := 1;
             Gam_Set_Timer_Game_Interval;
             Timer_Game.Enabled := true;

             iKeyPosition:=0;
             Timer_Key_Blink.Interval := 500;
             Timer_Key_Blink.Enabled := true;

             iLoopSparkBlink := 1;
             Timer_Spark_Blink.Interval := 50;
             Timer_Spark_Blink.Enabled := true;

             bController := true;
             bArrowKeyPressed := false;
           end
         else
           begin
             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 1;
             Timer_Game_Over.Enabled := true;
           end;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Score_BlinkTimer(Sender: TObject);
{ score panel blinks during "chance time"                                      }
begin
  Vid_Show_Panel_Score(iScore, Odd(iLoopScoreBlink));

  Inc(iLoopScoreBlink);

  if iLoopScoreBlink = 3 then
    iLoopScoreBlink := 1;
end;





procedure TForm1.Timer_Spark_BlinkTimer(Sender: TObject);
{ sparks blink when in (0,6) and (5,5)                                         }
begin
  aIm_enemies[0, 6].Visible := Odd(iLoopSparkBlink) and (aEnemies[0, 6] = 1);
  aIm_enemies[5, 5].Visible := (not Odd(iLoopSparkBlink)) and (aEnemies[5, 5] = 1);

  Inc(iLoopSparkBlink);

  if iLoopSparkBlink = 3 then
    iLoopSparkBlink := 1;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Form1.Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
